function [d, f, pred, cycle] = depthFirstSearch(adj_mat, start, directed)
% DFS Perform a depth-first search of the graph starting from 'start'.
% function [d, f, pred, cycle] = dfs(adj_mat, start, directed)
%
% Input:
% adj_mat(i,j)=1 iff i is connected to j.
% start is the root vertex of the dfs tree; if [], all nodes are searched
% directed = 1 if the graph is directed
%
% Output:
% d(i) is the time at which node i is first discovered.
% f(i) is the time at which node i is finished.
% pred(i) is the predecessor of i in the dfs tree.
% 'cycle' is true iff a (directed) cycle is found.
%
% See Cormen, Leiserson and Rivest, "An intro. to algorithms" 1994, p478.

n = length(adj_mat);

global white gray black color time_stamp
white = 0; gray = 1; black = 2;
color = white*ones(1,n);
time_stamp = 0;

global d f pred cycle
d = zeros(1,n);
f = zeros(1,n);
pred = zeros(1,n);
cycle = 0;

if ~isempty(start)
  dfs_visit(start, adj_mat, directed);
end
for u=1:n
  if color(u)==white
    dfs_visit(u, adj_mat, directed);
  end
end


%%%%%%%%%%

function dfs_visit(u, adj_mat, directed)

global white gray black color time_stamp d f pred cycle  

color(u) = gray;
time_stamp = time_stamp + 1;
d(u) = time_stamp;
if directed
  ns = children(adj_mat, u);
else
  ns = neighbors(adj_mat, u);
  ns = setdiff(ns, pred(u)); % don't go back to visit the guy who called you!
end
for v=ns(:)'
  switch color(v)
    case white, % not visited v before (tree edge)
     pred(v)=u;
     dfs_visit(v, adj_mat, directed);
   case gray, % back edge - v has been visited, but is still open
    cycle = 1;
   case black, % v has been visited, but is closed (forward or cross edge)
    assert(directed | cycle); % verify Thm 23.9 of CLR 
    % no-op
  end
end
color(u) = black;
time_stamp = time_stamp + 1;
f(u) = time_stamp;


